"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    }
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var React = require("react");
var _ = require("lodash");
var storm_react_diagrams_1 = require("storm-react-diagrams");
require("storm-react-diagrams/dist/style.min.css");
var LinkFactory_1 = require("./diagram/factories/LinkFactory");
var NodeFactory_1 = require("./diagram/factories/NodeFactory");
var PortFactory_1 = require("./diagram/factories/PortFactory");
var NodeModel_1 = require("./diagram/models/NodeModel");
var PortModel_1 = require("./diagram/models/PortModel");
var dagreUtils_1 = require("./utils/dagreUtils");
require("./styles/ReactDatabaseDiagram.css");
/**
 * A react component to render nice database diagram using storm-react-diagrams
 *
 * @class ReactDatabaseDiagram
 * @extends {React.Component<ReactDatabasDiagramProps>}
 */
var ReactDatabaseDiagram = /** @class */ (function (_super) {
    __extends(ReactDatabaseDiagram, _super);
    function ReactDatabaseDiagram(props) {
        var _this = _super.call(this, props) || this;
        _this.sceneNodes = new Map();
        _this.sceneLinks = [];
        _this.registerEngine = function () {
            _this.engine = new storm_react_diagrams_1.DiagramEngine();
            _this.engine.installDefaultFactories();
            _this.engine.registerPortFactory(new PortFactory_1.ReactDatabaseDiagramPortFactory("react-database-diagram", function () { return new PortModel_1.ReactDatabaseDiagramPortModel(); }));
            _this.engine.registerLinkFactory(new LinkFactory_1.ReactDatabaseDiagramLinkFactory());
            _this.engine.registerNodeFactory(new NodeFactory_1.ReactDatabaseDiagramNodeFactory());
            _this.model = new storm_react_diagrams_1.DiagramModel();
            _this.updateDiagram();
            _this.engine.setDiagramModel(_this.model);
        };
        // Add table to Scene only if it's not there already
        _this.addToScene = function (table, index, child) {
            var key = table.table_schema + "-" + table.table_name;
            if (_this.sceneNodes.has(key)) {
                return _this.sceneNodes.get(key);
            }
            var node = new NodeModel_1.ReactDatabaseDiagramNodeModel(table);
            _this.sceneNodes.set(key, node);
            if (child) {
                node.y = 250;
            }
            node.x = index * 230;
            _this.model.addNode(node);
            return node;
        };
        _this.getDistributedModel = function (engine, model) {
            var serialized = model.serializeDiagram();
            var distributedSerializedDiagram = dagreUtils_1.distributeElements(serialized);
            // Deserialize the model
            var deSerializedModel = new storm_react_diagrams_1.DiagramModel();
            deSerializedModel.deSerializeDiagram(distributedSerializedDiagram, engine);
            return deSerializedModel;
        };
        _this.registerEngine();
        return _this;
    }
    ReactDatabaseDiagram.prototype.componentDidMount = function () {
        this.engine.zoomToFit();
    };
    ReactDatabaseDiagram.prototype.updateDiagram = function () {
        var _this = this;
        var _a;
        var schema = this.props.schema;
        // Link each child node to its parent
        _.each(schema, function (table, index) {
            var parentPort = _this.addToScene(table, index).getPort("bottom");
            _.each(table.foreign_keys, function (tableName, indexChild) {
                var foreignTable = schema[_.findIndex(schema, { table_name: tableName.toTable })];
                var childPort = _this.addToScene(foreignTable, indexChild, true).getPort("top");
                var link = parentPort.link(childPort);
                _this.sceneLinks.push(link);
            });
        });
        (_a = this.model).addAll.apply(_a, this.sceneLinks);
        this.getDistributedModel(this.engine, this.model);
    };
    ReactDatabaseDiagram.prototype.render = function () {
        var config = this.props.config;
        return (React.createElement(storm_react_diagrams_1.DiagramWidget, __assign({ className: "react-database-diagram-canvas" }, config, { diagramEngine: this.engine })));
    };
    ReactDatabaseDiagram.defaultProps = {
        allowLooseLinks: false,
        allowCanvasTranslation: true,
        allowCanvasZoom: true,
        maxNumberPointsPerLink: 0,
        smartRouting: true,
    };
    return ReactDatabaseDiagram;
}(React.Component));
exports.default = ReactDatabaseDiagram;
//# sourceMappingURL=ReactDatabaseDiagram.js.map